#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// tslint:disable:no-var-requires
require("ts-node/register/transpile-only");
const path = __importStar(require("path"));
require("dotenv").config({
    path: process.env.ENV_FILE || path.join(process.cwd(), ".env")
});
const auth_app_1 = require("@octokit/auth-app");
const rest_1 = require("@octokit/rest");
const eslint_1 = require("./eslint");
const tslint_1 = require("./tslint");
const typescript_1 = require("./typescript");
const TSLINT_CHECK_APP_ID = 42099;
const yargs_1 = __importDefault(require("yargs"));
const git_helpers_1 = require("./git-helpers");
// tslint:disable-next-line: no-unused-expression
yargs_1.default
    .epilogue(`
This tool gets TypeScript, TSLint, or ESLint diagnostics and posts results as a "check run" to the given GitHub repository.

The following environment variables, corresponding to a GitHub app with 'checks:write' permission, are used to authenticate with the GitHub API:

GITHUB_APP_PRIVATE_KEY
GITHUB_APP_INSTALLATION_ID
GITHUB_APP_CLIENT_ID
GITHUB_APP_CLIENT_SECRET

They can also be provided in a ".env" file in the current working directory.`)
    .option("repo", {
    describe: 'The github repository, "owner/repo"',
    type: "string"
})
    .options("sha", {
    describe: "The git sha to which to post check results. Defaults to HEAD",
    type: "string"
})
    .options("label", {
    describe: "A label for this check run",
    type: "string"
})
    .command("tsc <tsconfig>", "Check TypeScript errors", builder => builder.positional("tsconfig", {
    describe: "Path to the TypeScript project configuration file",
    type: "string",
    default: path.join(process.cwd(), "tsconfig.json")
}), argv => {
    runCheck("Typescript", () => typescript_1.typescriptCheck(argv.tsconfig), {
        ...argv,
        location: argv.tsconfig
    });
})
    .command("tslint <tsconfig>", "Check TSLint errors", builder => builder.positional("tsconfig", {
    describe: "Path to the TypeScript project configuration file",
    type: "string",
    default: path.join(process.cwd(), "tsconfig.json")
}), argv => runCheck("TSLint", () => tslint_1.tslintCheck({ tsConfigFile: argv.tsconfig }), {
    ...argv,
    location: argv.tsconfig
}))
    .command("eslint <directory>", "Check ESLint errors", builder => builder
    .positional("directory", {
    describe: "Location of files to lint",
    type: "string",
    demand: true
})
    .option("overrideConfig", {
    describe: "ESLint configuration overrides as a JSON string",
    type: "string",
    default: "{}"
}), argv => runCheck("ESLint", () => eslint_1.eslintCheck([argv.directory || "."], JSON.parse(argv.overrideConfig)), {
    ...argv,
    location: argv.directory || "."
})).argv;
async function runCheck(commandName, check, options) {
    let gh;
    if (options.repo) {
        const [owner, repo] = options.repo.split("/");
        if (!owner || !repo) {
            console.error(`Invalid --repo argument ${options.repo}. Expected "owner/repo".`);
            process.exit(1);
        }
        gh = {
            github: await authenticate(),
            owner,
            repo,
            sha: options.sha
        };
    }
    const commandAndLabel = options.label
        ? `${commandName} - ${options.label}`
        : commandName;
    const baseDir = git_helpers_1.getGitRepositoryDirectoryForFile(options.location);
    let ghCheck;
    if (gh) {
        ghCheck = await gh.github.checks.create({
            owner: gh.owner,
            repo: gh.repo,
            head_sha: gh.sha || git_helpers_1.getGitSHA(baseDir),
            name: commandAndLabel,
            status: "in_progress"
        });
        console.log(`Created check ${ghCheck.data.id} (${ghCheck.data.url})`);
    }
    const linterResult = await check();
    const annotations = linterResult.annotations.map(a => ({
        ...a,
        path: path.relative(baseDir, a.path) // patch file paths to be relative to git root
    }));
    const summary = `${linterResult.errorCount} errors, ${linterResult.warningCount} warnings.`;
    const conclusion = linterResult.errorCount + linterResult.warningCount > 0
        ? "failure"
        : "success";
    console.log(`${commandAndLabel}: ${summary}`);
    console.log(linterResult.consoleOutput);
    if (gh && ghCheck) {
        for (let updateCount = 0; updateCount === 0 || annotations.length > 0; updateCount++) {
            const batch = annotations.splice(0, 50);
            const update = await gh.github.checks.update({
                check_run_id: ghCheck.data.id,
                owner: gh.owner,
                repo: gh.repo,
                output: {
                    annotations: batch,
                    summary,
                    title: commandAndLabel
                },
                conclusion
            });
            console.log(`Updated check ${update.data.id} with ${batch.length} annotations.`);
        }
    }
    if (conclusion === "failure") {
        process.exit(1);
    }
}
async function authenticate() {
    const auth = auth_app_1.createAppAuth({
        id: TSLINT_CHECK_APP_ID,
        privateKey: process.env.GITHUB_APP_PRIVATE_KEY || "",
        installationId: Number(process.env.GITHUB_APP_INSTALLATION_ID),
        clientId: process.env.GITHUB_APP_CLIENT_ID,
        clientSecret: process.env.GITHUB_APP_CLIENT_SECRET
    });
    // Retrieve installation access token
    const installationAuthentication = await auth({ type: "installation" });
    return new rest_1.Octokit({
        auth: installationAuthentication.token
    });
}
